/*
 * trying I2C access through the bcm2835 library
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>

#include <bcm2835.h>

#define MCP4725_ADDRESS 0x62

int main() {
  uint8_t retCode;
  char i2cData[2];
  struct timeval tv1,tv2;
  struct timezone tz;
  
  if ((retCode=bcm2835_init()) < 0) {
    printf("bcm2835I2C: Could not open bcm2835 library! Are you root?\n");
    return -1;
  }
  else
    printf("bcm2835I2C: bcm2835 library successfully opened!\n");

  /*
    now open the I2C bus
  */
  if ((retCode=bcm2835_i2c_begin()) < 0) {
    printf("bcm2835I2C: error when opening the I2C bus\n");
    return -1;
  }
  else
    printf("bcm2835I2C: I2C bus successfully opened\n");

  bcm2835_i2c_setSlaveAddress(MCP4725_ADDRESS);
  bcm2835_i2c_setClockDivider(BCM2835_I2C_CLOCK_DIVIDER_148);
  printf("bcm2835I2C: slave address set to 0x%02x\n",MCP4725_ADDRESS);
  printf("bcm2835I2C: bit rate set to 1.689 Mbps\n");
  
  for(;;) {
    i2cData[0]=0x0f;
    i2cData[1]=0xff;
    /*                                                                          
      get the system time before the call                                       
    */
    if (gettimeofday(&tv1,&tz) < 0)
      printf("bcm2835I2C: Could not get system time\n");
    
    bcm2835_i2c_write(i2cData,2);
         printf("bcm2835I2C: Could not get system time\n");
    /*                                                                          
      get the system time immediately after the call                            
    */
    if (gettimeofday(&tv2,&tz) < 0)
      printf("bcm2835I2C: Could not get system time\n");

    printf("tv1 sec: %d, tv2 sec: %d, tv1 usec %d, tv2 usec %d\n",
           (int)tv1.tv_sec,(int)tv2.tv_sec,(int)tv1.tv_usec,(int)tv2.tv_usec);

    if (tv2.tv_sec == tv1.tv_sec)
      printf("bcm2835I2C: duration: %ld\n",(long)(tv2.tv_usec-tv1.tv_usec));
 
    sleep(2);

    i2cData[0]=0x00;
    i2cData[1]=0x00;
    bcm2835_i2c_write(i2cData,2);
    sleep(2);
  }

  bcm2835_i2c_end();
  if ((retCode=bcm2835_close()) < 0)
    printf("bcm2835I2C: error when closing the bcm2835 library\n");
  return 0;
}
