/* dacTest: A simple test program to see if we can access the mcp4275 DAC
 *    it sends a square wave (0,0xfff) to the DAC at a frequency or 1/4 Hz
 *    which allows to easily observe it with a multi-meter
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <pigpio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdbool.h>
#include <signal.h>

#define I2C_BUS          1
#define I2C_FLAGS        0
#define MCP4725_ADDRESS  0x62
/* max. dac value for 12 bit DAC */
#define MAX_DAC_VALUE    0x0fff
#define OK               0

bool mcp4725Debug = false;
int handle;

void mcp4725SetDebug(bool onOff) {
  mcp4725Debug=onOff;
}

void dacExit(int dummy) {
  /*
    clean everything and exit 
  */
  printf("dacTest: Cleaning up\n");
  i2cClose(handle);
  gpioTerminate();
  exit(0);
}

int main(int argc, char **argv) {
  int retCode=-1;
  short dacValue;
  unsigned char reg,value;
  mcp4725SetDebug(true);
  
  /*
    initialize pigpio library
  */
  if ((retCode=gpioInitialise()) < 0) {
    printf("dacTest: Could not open pigpio library\n");
    printf("retCode: %d: ",retCode);
    return -1;
  }
  
  if ((retCode = i2cOpen(I2C_BUS,MCP4725_ADDRESS,I2C_FLAGS)) < 0) {
    printf("dacTest: Cannot open I2C bus %d at address %02x\n",
	   I2C_BUS,MCP4725_ADDRESS);
    printf("Error code: %d\n",retCode);
    return (-1);
  }
  if (mcp4725Debug)
    printf("dacTest: pigpio successfully initialized\n");
  
  handle = retCode;
  /*
    capture ^C to gracefully exit the program
  */
  signal(SIGINT,dacExit);
  
  /*
    send a square wave
  */
  for(;;) {
    dacValue = MAX_DAC_VALUE;
    reg = dacValue >> 8;
    value = dacValue&0xff;
    retCode = i2cWriteByteData(handle,reg,value);
    if (retCode != OK) {
      printf("dacTest: could not write to the dac register errCode= %d\n",retCode);
      return(-1);
    }     
    else
      printf("dac: The value %d has been sent to the DAC\n",dacValue);
    sleep(2);
    dacValue =0;
    reg = dacValue >> 8;
    value = dacValue&0xff;    
    retCode = i2cWriteByteData(handle,reg,value);
    if (retCode != OK) {
      printf("dacTest: could not write to the dac register errCode= %d\n",retCode);
      return(-1);
    }     
    else
      printf("dac: The value %d has been sent to the DAC\n",dacValue);
    sleep(2);
  }
 
  return OK;
}
