/*
 * pigpioPrintErrMsg(): prints a humanly readable error message
 *    from a pigpio error code
 *    Copyright U. Raich Jan. 2017
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <pigpiod_if2.h>
#include <stdio.h>

void printPigpioErr(pigifError_t errCode) {
  switch (errCode) {
  case pigif_bad_send:
    printf("pigpio: Could not send\n");
    break;
  
  case pigif_bad_recv:
    printf("pigpio; Receiving failed\n");
    break;
  
  case pigif_bad_getaddrinfo:
    printf("pigpio: Get address info failed\n");
    break;
  
  case pigif_bad_connect:
    printf("pigpio: Could not connect to the daemon\n");
    break;
  
  case pigif_bad_socket:
    printf("pigpio: Could not get socket\n");
    break;
  
  case pigif_bad_noib:
    printf("pigpio: bad noib\n");
    break;
  
  case pigif_duplicate_callback:
    printf("pigpio: Callback was defined previously\n");
    break;
  
  case pigif_bad_malloc:
    printf("pigpio: Bad malloc\n");
    break;
  
  case pigif_bad_callback:
    printf("pigpio: Bad callback\n");
    break;
  
  case pigif_notify_failed:
    printf("pigpio: Notify failed\n");
    break;
  
  case pigif_callback_not_found:
    printf("pigpio: Callback not found\n");
    break;
  
  case pigif_unconnected_pi:
    printf("pigpio: Pi is not connected\n");
    break;
  
  case pigif_too_many_pis:
    printf("pigpio: There are too many Pis\n");
    break;
  }
}

void pigpioPrintI2CError(int err) {
  switch(err) {
  case PI_BAD_I2C_BUS:
    printf("pigpio i2c: bad I2C bus\n");
    break;
  case PI_BAD_I2C_ADDR:
    printf("pigpio i2c: bad I2C address\n");
    break;
  case PI_BAD_FLAGS:
    printf("pigpio i2c: bad I2C flags\n");
    break;
  case PI_NO_HANDLE:
     printf("pigpio i2c: no handle\n");
     break;
  case PI_I2C_OPEN_FAILED:
     printf("pigpio i2c: I2C open failed\n");
     break;   
  default:
    printf("pigpio i2c: strange ... I don't know this error!\n");
    break;
  }
}
